/**
 * 
 */
package org.gcube.documentstore.persistence;

import java.util.Calendar;
import java.util.concurrent.TimeUnit;

import org.gcube.accounting.datamodel.UsageRecord;
import org.gcube.accounting.datamodel.basetypes.TestUsageRecord;
import org.gcube.accounting.testutility.StressTestUtility;
import org.gcube.accounting.testutility.TestOperation;
import org.gcube.common.scope.api.ScopeProvider;
import org.gcube.documentstore.exception.InvalidValueException;
import org.gcube.documentstore.persistence.FallbackPersistenceBackend;
import org.gcube.documentstore.persistence.PersistenceBackend;
import org.gcube.documentstore.persistence.PersistenceBackendFactory;
import org.junit.Assert;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Luca Frosini (ISTI - CNR) http://www.lucafrosini.com/
 *
 */
public class PersistenceBackendTest {
	
	private static Logger logger = LoggerFactory.getLogger(PersistenceBackendTest.class);
	
	public static final String[] SCOPES = new String[]{"/gcube", "/gcube/devNext"};
	public static final String GCUBE_SCOPE = SCOPES[0];
	public static final String GCUBE_DEVNEXT_SCOPE = SCOPES[1];
	
	public static final long timeout = 5000;
	public static final TimeUnit timeUnit = TimeUnit.MILLISECONDS;
	
	public static PersistenceBackend getPersistence(){
		ScopeProvider.instance.set(GCUBE_DEVNEXT_SCOPE);
		PersistenceBackendFactory.setFallbackLocation(null);
		return PersistenceBackendFactory.getPersistenceBackend(ScopeProvider.instance.get());
	}
	
	@Test
	public void singleTestNoScope() throws Exception {
		PersistenceBackendFactory.setFallbackLocation(null);
		final PersistenceBackend persistence = PersistenceBackendFactory.getPersistenceBackend(null);
		Assert.assertTrue(persistence instanceof FallbackPersistenceBackend);
		StressTestUtility.stressTest(new TestOperation() {
			@Override
			public void operate(int i) {
				UsageRecord usageRecord;
				try {
					usageRecord = TestUsageRecord.createTestServiceUsageRecordExplicitScope();
					persistence.accountValidateAggregate(usageRecord, true, false);
				} catch (InvalidValueException e) {
					throw new RuntimeException(e);
				}
			}
		}, 1);
		
		persistence.flush(timeout, timeUnit);
	}
	
	@Test
	public void singleTest() throws Exception {
		final PersistenceBackend persistence = getPersistence();
		StressTestUtility.stressTest(new TestOperation() {
			@Override
			public void operate(int i) {
				UsageRecord usageRecord = TestUsageRecord.createTestServiceUsageRecordAutomaticScope();
				persistence.accountValidateAggregate(usageRecord, true, false);
			}
		}, 1);
		
		persistence.flush(timeout, timeUnit);
	}
	
	@Test
	public void stressTestNoAggregation() throws Exception {
		final PersistenceBackend persistence = getPersistence();
		StressTestUtility.stressTest(new TestOperation() {
			@Override
			public void operate(int i) {
				UsageRecord usageRecord = TestUsageRecord.createTestServiceUsageRecordAutomaticScope();
				persistence.accountValidateAggregate(usageRecord, true, false);
			}
		});
	}
	
	@Test
	public void stressTestWithAggregation() throws Exception {
		final PersistenceBackend persistence = getPersistence();
		
		StressTestUtility.stressTest(new TestOperation() {
			@Override
			public void operate(int i) throws Exception {
				UsageRecord usageRecord = TestUsageRecord.createTestServiceUsageRecordAutomaticScope();
				persistence.account(usageRecord);
			}
		});
		persistence.flush(timeout, timeUnit);
	}
	
	
	@Test
	public void testScopeRecheck() throws Exception {
		ScopeProvider.instance.set("/fakeScope");
		PersistenceBackendFactory.setFallbackLocation(null);
		logger.debug("Going to check First Time");
		PersistenceBackend first = PersistenceBackendFactory.getPersistenceBackend(ScopeProvider.instance.get());
		logger.debug("First {} : {}", PersistenceBackend.class.getSimpleName(), first);
		Long firstCheck = PersistenceBackendFactory.getFallbackLastCheck(ScopeProvider.instance.get());
		logger.debug("First Check Time {}", firstCheck);
		
		long startTime = Calendar.getInstance().getTimeInMillis();
		long endTime = startTime;
		
		while(endTime <=  (startTime + (PersistenceBackendFactory.FALLBACK_RETRY_TIME + 100))){
			endTime = Calendar.getInstance().getTimeInMillis();
		}

		logger.debug("Going to check Second Time");
		PersistenceBackend second = PersistenceBackendFactory.getPersistenceBackend(ScopeProvider.instance.get());
		logger.debug("Second {} : {}", PersistenceBackend.class.getSimpleName(), second);
		Long secondCheck = PersistenceBackendFactory.getFallbackLastCheck(ScopeProvider.instance.get());
		logger.debug("Second Check Time {}", secondCheck);
		
		Assert.assertNotEquals(firstCheck, secondCheck);
		Assert.assertEquals(first, second);
	}
	
}
